"""
class cybercops encoding.
"""
#import codecs
from _errors import EncodingError

class Encode(object):
    """
    This class is converts the character-code of the return data.
    """
    
    __UTF8  = 'utf-8'
    __EUCJP = 'euc-jp'
    __SJIS  = 'cp932'
    
    __INTERNAL_MAP = {
        'UTF-8'     : __UTF8,
        'EUCJP-WIN' : __EUCJP, 'EUC-JP'    : __EUCJP,
        'SJIS-WIN'  : __SJIS,  'Shift_JIS' : __SJIS
    }
    
    __DISPATCH_MAP = {
        'UTF-8'     : 'UTF-8',
        'EUC-JP'    : 'EUCJP-WIN',
        'SJIS'      : 'SJIS-WIN'
    }
    
    __UNICODEMAP_U2E = (
        (u'\uff5e', u'\u301c'), # U+FF5E (FULLWIDTH TILDE)       , U+301C (WAVE DASH)
        (u'\uff0d', u'\u2212'), # U+FF0D (FULLWIDTH HYPHEN-MINUS), U+2212 (MINUS SIGN)
        (u'\u2225', u'\u2016'), # U+2225 (PARALLEL TO)           , U+2016 (DOUBLE VERTICAL LINE)
        (u'\uffe0', u'\u00a2'), # U+FFE0 (FULLWIDTH CENT SIGN)   , U+00A2 (CENT SIGN)
        (u'\uffe1', u'\u00a3'), # U+FFE1 (FULLWIDTH POUND SIGN)  , U+00A3 (POUND SIGN)
        (u'\uffe2', u'\u00ac')  # U+FFE2 (FULLWIDTH NOT SIGN)    , U+00AC (NOT SIGN)
    )
    
    __UNICODEMAP_S2E = (
        (u'\uff5e', u'\u301c'), # U+FF5E (FULLWIDTH TILDE)       , U+301C (WAVE DASH)
        (u'\uff0d', u'\u2212'), # U+FF0D (FULLWIDTH HYPHEN-MINUS), U+2212 (MINUS SIGN)
        (u'\u2225', u'\u2016'), # U+2225 (PARALLEL TO)           , U+2016 (DOUBLE VERTICAL LINE)
        (u'\uffe0', u'\u00a2'), # U+FFE0 (FULLWIDTH CENT SIGN)   , U+00A2 (CENT SIGN)
        (u'\uffe1', u'\u00a3'), # U+FFE1 (FULLWIDTH POUND SIGN)  , U+00A3 (POUND SIGN)
        (u'\uffe2', u'\u00ac')  # U+FFE2 (FULLWIDTH NOT SIGN)    , U+00AC (NOT SIGN)
    )
    
    def get_dispatch_encoding(self, charset):
        if self.__DISPATCH_MAP.has_key(charset):
            return self.__DISPATCH_MAP[charset]
        else:
            raise EncodingError('There is no `%s` in effective.' % charset)
    get_dispatch_encoding = classmethod(get_dispatch_encoding)
    
    def get_encoding(self, charset):
        if self.__INTERNAL_MAP.has_key(charset):
            return self.__INTERNAL_MAP[charset]
        else:
            raise EncodingError('There is no `%s` in effective.' % charset)
    get_encoding = classmethod(get_encoding)
    
    
    def convert(self, str, after, before):
        if before is after:
            return str
        elif after is self.__EUCJP:
            us = unicode(str, before)
            if before is self.__UTF8:
                charmap = self.__UNICODEMAP_U2E
            if before is self.__SJIS:
                charmap = self.__UNICODEMAP_S2E
            for (bf, af) in charmap:
                us = us.replace(bf, af)
            return us.encode(after)
        elif after is self.__UTF8 and before is self.__EUCJP:
            us = unicode(str, before)
            for (af, bf) in self.__UNICODEMAP_U2E:
                us = us.replace(bf, af)
            return us.encode(after)
        else:
            return unicode(str, before).encode(after)
    convert = classmethod(convert)

if __name__ == '__main__':
    s = Encode()
    for n in dir(s): print n
