"""
module CyberCops.parser
"""
from _result import Result
from _encode import Encode
from _errors import ParseError
import re

class Parser(object):
    
    __XML_DECLARATION = re.compile(
        '<\?xml version="1\.0" encoding="([\w\-]+)"'
    )
    __XML_RESULT_NODE = re.compile(
        '<result error="(\d+)" words="(\d+)" count="(\d+)" userid="([\w\-]*?)" errormsg="(.*?)"'
    )
    __XML_WORD_NODE = re.compile(
        '<word level="(\d)" count="(\d+)">(.+?)<\/word>'
    )
    
    
    def __init__(self, charset):
        self.__charset = Encode.get_encoding(charset)
        self.__xml_charset = None
    
    def parse(self, response):
        cnt = 0
        result = Result.Summary()
        for line in response.splitlines():
            cnt += 1
            if cnt is 1:
                m = self.__XML_DECLARATION.search(line)
                if not m:
                    raise ParseError('The XML declaration is not found.')
                result._Summary__encoding = m.group(1)
                self.__xml_charset = Encode.get_encoding(result._Summary__encoding)
            elif cnt is 2:
                m = self.__XML_RESULT_NODE.search(line)
                if not m:
                    raise ParseError('The Result node is not found from the XML.')
                result._Summary__error    = int(m.group(1))
                result._Summary__words    = int(m.group(2))
                result._Summary__count    = int(m.group(3))
                result._Summary__userid   = m.group(4)
                result._Summary__errormsg = m.group(5)
                if result._Summary__error is not 0:
                    break
            else:
                m = self.__XML_WORD_NODE.search(line)
                if m:
                    description = Result.Description()
                    description._Description__level = int(m.group(1))
                    description._Description__count = int(m.group(2))
                    description._Description__word  = \
                        Encode.convert(m.group(3), self.__charset, self.__xml_charset)
                    result._Summary__items.append(description)
        return result
    
if __name__ == '__main__':
    cls = Parser('UTF-8')
    for i in dir(cls): print i
